/* Generated automatically. */
static const char configuration_arguments[] = "/usr/obj/ports/xtensa-esp32-elf-gcc-bootstrap-8.4.0.2021.2/gcc-esp-2021r2/configure --exec-prefix=/usr/local/xtensa-esp32-elf/bootstrap --with-as=/usr/local/xtensa-esp32-elf/bin/xtensa-esp32-elf-as --with-ld=/usr/local/xtensa-esp32-elf/bin/xtensa-esp32-elf-ld --with-gmp=/usr/local --with-isl=/usr/local --target=xtensa-esp32-elf --prefix=/usr/local/xtensa-esp32-elf --disable-shared --with-newlib --enable-threads=no --disable-__cxa_atexit --disable-libgomp --disable-libmudflap --disable-nls --disable-bootstrap --enable-languages=c,c++ --disable-lto --disable-libstdcxx-verbose --disable-option-checking --without-long-double-128 --with-system-zlib --enable-static=yes";
static const char thread_model[] = "single";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { NULL, NULL} };
